@@ -13,8 +13,9 @@ import hashlib
 import heapq
 import sys
 from collections import OrderedDict, abc
+from collections.abc import Sequence
 from functools import lru_cache
-from typing import TYPE_CHECKING, List, Optional, Sequence, Tuple, Type, TypeVar, Union
+from typing import TYPE_CHECKING, List, Optional, TypeVar, Union
 
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import int_from_bytes
@@ -51,8 +52,12 @@ ONE_FROM_MANY_LABEL = calc_label_from_name("one more from many()")
 T = TypeVar("T")
 
 
+def identity(v: T) -> T:
+    return v
+
+
 def check_sample(
-    values: Union[Type[enum.Enum], Sequence[T]], strategy_name: str
+    values: Union[type[enum.Enum], Sequence[T]], strategy_name: str
 ) -> Sequence[T]:
     if "numpy" in sys.modules and isinstance(values, sys.modules["numpy"].ndarray):
         if values.ndim != 1:
@@ -99,15 +104,14 @@ class Sampler:
        shrinking the chosen element.
     """
 
-    table: List[Tuple[int, int, float]]  # (base_idx, alt_idx, alt_chance)
+    table: list[tuple[int, int, float]]  # (base_idx, alt_idx, alt_chance)
+
+    def __init__(self, weights: Sequence[float], *, observe: bool = True):
+        self.observe = observe
 
-    def __init__(self, weights: Sequence[float]):
         n = len(weights)
-
         table: "list[list[int | float | None]]" = [[i, None, None] for i in range(n)]
-
         total = sum(weights)
-
         num_type = type(total)
 
         zero = num_type(0)  # type: ignore
@@ -155,7 +159,7 @@ class Sampler:
         while small:
             table[small.pop()][2] = zero
 
-        self.table: "List[Tuple[int, int, float]]" = []
+        self.table: "list[tuple[int, int, float]]" = []
         for base, alternate, alternate_chance in table:  # type: ignore
             assert isinstance(base, int)
             assert isinstance(alternate, int) or alternate is None
@@ -167,8 +171,15 @@ class Sampler:
                 self.table.append((base, alternate, alternate_chance))
         self.table.sort()
 
-    def sample(self, data: "ConjectureData", forced: Optional[int] = None) -> int:
-        data.start_example(SAMPLE_IN_SAMPLER_LABEL)
+    def sample(
+        self,
+        data: "ConjectureData",
+        *,
+        forced: Optional[int] = None,
+        fake_forced: bool = False,
+    ) -> int:
+        if self.observe:
+            data.start_example(SAMPLE_IN_SAMPLER_LABEL)
         forced_choice = (  # pragma: no branch # https://github.com/nedbat/coveragepy/issues/1617
             None
             if forced is None
@@ -179,7 +190,10 @@ class Sampler:
             )
         )
         base, alternate, alternate_chance = data.choice(
-            self.table, forced=forced_choice
+            self.table,
+            forced=forced_choice,
+            fake_forced=fake_forced,
+            observe=self.observe,
         )
         forced_use_alternate = None
         if forced is not None:
@@ -189,8 +203,14 @@ class Sampler:
             forced_use_alternate = forced == alternate and alternate_chance > 0
             assert forced == base or forced_use_alternate
 
-        use_alternate = data.draw_boolean(alternate_chance, forced=forced_use_alternate)
-        data.stop_example()
+        use_alternate = data.draw_boolean(
+            alternate_chance,
+            forced=forced_use_alternate,
+            fake_forced=fake_forced,
+            observe=self.observe,
+        )
+        if self.observe:
+            data.stop_example()
         if use_alternate:
             assert forced is None or alternate == forced, (forced, alternate)
             return alternate
@@ -200,7 +220,7 @@ class Sampler:
 
 
 INT_SIZES = (8, 16, 32, 64, 128)
-INT_SIZES_SAMPLER = Sampler((4.0, 8.0, 1.0, 1.0, 0.5))
+INT_SIZES_SAMPLER = Sampler((4.0, 8.0, 1.0, 1.0, 0.5), observe=False)
 
 
 class many:
@@ -223,6 +243,8 @@ class many:
         average_size: Union[int, float],
         *,
         forced: Optional[int] = None,
+        fake_forced: bool = False,
+        observe: bool = True,
     ) -> None:
         assert 0 <= min_size <= average_size <= max_size
         assert forced is None or min_size <= forced <= max_size
@@ -230,23 +252,32 @@ class many:
         self.max_size = max_size
         self.data = data
         self.forced_size = forced
+        self.fake_forced = fake_forced
         self.p_continue = _calc_p_continue(average_size - min_size, max_size - min_size)
         self.count = 0
         self.rejections = 0
         self.drawn = False
         self.force_stop = False
         self.rejected = False
+        self.observe = observe
+
+    def stop_example(self):
+        if self.observe:
+            self.data.stop_example()
+
+    def start_example(self, label):
+        if self.observe:
+            self.data.start_example(label)
 
     def more(self) -> bool:
         """Should I draw another element to add to the collection?"""
         if self.drawn:
-            self.data.stop_example(discard=self.rejected)
+            self.stop_example()
 
         self.drawn = True
         self.rejected = False
 
-        self.data.start_example(ONE_FROM_MANY_LABEL)
-
+        self.start_example(ONE_FROM_MANY_LABEL)
         if self.min_size == self.max_size:
             # if we have to hit an exact size, draw unconditionally until that
             # point, and no further.
@@ -265,14 +296,17 @@ class many:
             elif self.forced_size is not None:
                 forced_result = self.count < self.forced_size
             should_continue = self.data.draw_boolean(
-                self.p_continue, forced=forced_result
+                self.p_continue,
+                forced=forced_result,
+                fake_forced=self.fake_forced,
+                observe=self.observe,
             )
 
         if should_continue:
             self.count += 1
             return True
         else:
-            self.data.stop_example()
+            self.stop_example()
             return False
 
     def reject(self, why: Optional[str] = None) -> None:
